/**
 * Copyright (C) 2008 Daniele Dellafiore <daniele.dellafiore@gmail.com>
 * 
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/MPL-1.1.html
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific language governing rights and
 * limitations under the License.
 */

package net.phaedra.commons.utils;

import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

public class DateUtil {

   static SimpleDateFormat yearMonthDayFormat = new SimpleDateFormat("yyyy-MM-dd");

   public static String formatYearMonthDay(Timestamp timestamp) {
      return yearMonthDayFormat.format(timestamp);
   }

   public static Timestamp createTimestamp(int year, int month, int day) {
      return toTimestamp(createDate(year, month, day));
   }

   private static Date createDate(int year, int month, int day) {
      return new GregorianCalendar(year, month - 1, day).getTime();
   }

   public static Timestamp parseYearMonthDay(String dateAsText) throws ParseException {
      Date date = yearMonthDayFormat.parse(dateAsText);
      return toTimestamp(date);
   }

   private static Timestamp toTimestamp(Date date) {
      return new Timestamp(date.getTime());
   }

   public static Date buildDate(int year, int month, int day, int hour, int minute) {

      Calendar calendar = createCalendarFromDate(year, month, day, hour, minute);

      Date date = calendar.getTime();

      return date;
   }

   private static Calendar createCalendarFromDate(int year, int month, int day, int hour, int minute) {
      Calendar calendar = Calendar.getInstance();
      calendar.set(year, month - 1, day);

      calendar.set(Calendar.HOUR_OF_DAY, hour);
      calendar.set(Calendar.MINUTE, minute);
      calendar.set(Calendar.SECOND, 0);
      calendar.set(Calendar.MILLISECOND, 0);
      return calendar;
   }

   public static Date buildDate(int year, int month, int day) {
      return buildDate(year, month, day, 0, 0);
   }

   public static int daysBetween(Date startDate, Date endDate) {
      return (int) ((endDate.getTime() - startDate.getTime() + DateFormatUtil.ONE_HOUR) / (DateFormatUtil.ONE_HOUR * 24));
   }

   public static Date addToDate(Date startDate, int year, int month, int day) {
      return calculateNewDate(startDate, year, month, day);
   }

   private static Date calculateNewDate(Date startDate, int year, int month, int day) {
      long dateTime = startDate.getTime();
      Calendar calendar = Calendar.getInstance();
      calendar.setTimeInMillis(dateTime);
      calendar.add(calendar.MONTH, month);
      calendar.add(calendar.YEAR, year);
      calendar.add(calendar.DAY_OF_MONTH, day);
      return calendar.getTime();
   }

   public static Date subFromDate(Date startDate, int year, int month, int day) {
      return calculateNewDate(startDate, -year, -month, -day);
   }

}
