package net.phaedra.commons.utils;

import java.util.Arrays;

import jin.collection.util.ReflectionUtil;

public class EntityParser {

   private final Class destinationBean;
   private final String[] properties;

   public EntityParser(final Class destinationBean, final String... properties) {
      this.destinationBean = destinationBean;
      this.properties = properties;
   }

   public Object read(final String... input) throws Exception {
      final int difference = properties.length - input.length;
      if (difference > 0) {
         throw new EntityParseException("number of properties differs from number of input fields by " + difference
               + " | Properties: " + Arrays.toString(properties) + " | Input: " + Arrays.toString(input));
      }
      final Object result = destinationBean.newInstance();
      for (int i = 0; i < properties.length; i++) {
         final String property = properties[i];
         try {
            ReflectionUtil.setPrivateField(result, property, input[i]);
         } catch (final IllegalArgumentException e) {
            try {
               BeanUtils.writeProperty(destinationBean, property, input[i]);
            } catch (final Exception e2) {
               throw new IllegalArgumentException("cannot write property:" + property + " on destination bean: "
                     + destinationBean, e2);
            }
         }
      }
      return result;
   }
}
