package net.phaedra.commons.utils;

public class MemoryPersistanceValueObject {
   private double milliSecondsToLive;
   private Object persistedObject;
   private long lastUpdateMillis;
   private Object defaultReturnValue;

   public MemoryPersistanceValueObject() {
   }

   public MemoryPersistanceValueObject(int seconds, String defaultValue) {
      this.defaultReturnValue = defaultValue;
      setSecondsToLive(seconds);
   }

   public void persist(Object object) {
      this.persistedObject = object;
      this.lastUpdateMillis = System.currentTimeMillis();
   }

   public Object retrieveObject() {
      if (isExpired()) {
         persistedObject = getDefaultReturnValue();
      }
      return persistedObject;
   }

   public boolean isExpired() {
      long elapsed = System.currentTimeMillis() - lastUpdateMillis;
      return elapsed > milliSecondsToLive;
   }

   public void setMilliSecondsToLive(double d) {
      this.milliSecondsToLive = d;
   }

   public void setSecondsToLive(double d) {
      this.milliSecondsToLive = d * 1000;
   }

   public void setDefaultReturnValue(Object defaultReturnValue) {
      this.defaultReturnValue = defaultReturnValue;
   }

   private Object getDefaultReturnValue() {
      return defaultReturnValue;
   }

   public void start(double time) {
      setMilliSecondsToLive(time);
      persist("");
   }
}
