package net.phaedra.commons.values;

import java.io.Serializable;
import java.text.DecimalFormat;

import org.apache.commons.lang.math.IntRange;

public class Ora implements Comparable, Serializable {

   private static final IntRange VALID_HOUR = new IntRange(0, 23);
   private static final IntRange VALID_MINUTE = new IntRange(0, 59);
   public final int hour;
   public final int minutes;

   public Ora(String ora) {
      this(Integer.parseInt(ora.split(":")[0]), Integer.parseInt(ora.split(":")[1]));
   }

   public Ora(int hour, int minute) {
      if (!VALID_HOUR.containsInteger(hour)) {
         throw new IllegalArgumentException("" + hour);
      }
      if (!VALID_MINUTE.containsInteger(minute)) {
         throw new IllegalArgumentException("" + minute);
      }
      this.hour = hour;
      this.minutes = minute;
   }

   private Ora(int periodInMinutes) {
      this.hour = periodInMinutes / 60;
      this.minutes = periodInMinutes % 60;
   }

   public String asString() {
      DecimalFormat decimalFormat = new DecimalFormat("00");
      return decimalFormat.format(hour) + ":" + decimalFormat.format(minutes);
   }

   public int toMinutes() {
      return hour * 60 + minutes;
   }

   public int compareTo(Object o) {
      return toMinutes() - ((Ora) o).toMinutes();
   }

   public boolean isAfter(Ora ora, int tolleranza) {
      int nowInMinutes = toMinutes();
      return nowInMinutes > ora.toMinutes() && nowInMinutes - ora.toMinutes() > tolleranza;
   }

   public boolean isBefore(Ora oraInizio, int tolleranza) {
      return !isAfter(oraInizio, tolleranza);
   }

   public Ora subtractMinutes(int delta) {
      return addMinutes(-delta);
   }

   public Ora addMinutes(int delta) {
      return new Ora(toMinutes() + delta);
   }

   boolean isBeforeOrEquals(Ora ora) {
      return isBefore(ora, 0) || equals(ora);
   }

   boolean isAfterOrEquals(Ora ora) {
      return isAfter(ora, 0) || equals(ora);
   }

   public String getText() {
      return asString();
   }

   @Override
   public int hashCode() {
      final int prime = 31;
      int result = 1;
      result = prime * result + this.hour;
      result = prime * result + this.minutes;
      return result;
   }

   @Override
   public boolean equals(Object obj) {
      if (this == obj) {
         return true;
      }
      if (obj == null) {
         return false;
      }
      if (getClass() != obj.getClass()) {
         return false;
      }
      Ora other = (Ora) obj;
      if (this.hour != other.hour) {
         return false;
      }
      if (this.minutes != other.minutes) {
         return false;
      }
      return true;
   }

   @Override
   public String toString() {
      return asString();
   }

}
