package net.phaedra.commons.values;

import java.io.Serializable;
import java.math.BigDecimal;

import org.apache.commons.lang.math.IntRange;

public class Percentuale implements Serializable {

   private static final BigDecimal CENTO = new BigDecimal(100);
   private final int percentuale;

   public Percentuale(int percentuale) {
      this.percentuale = percentuale;
      IntRange range = new IntRange(0, 100);

      if (!range.containsInteger(percentuale)) {
         throw new IllegalArgumentException("Valore percentuale inatteso: " + percentuale);
      }
   }

   public Percentuale(BigDecimal bigDecimal) {
      this(bigDecimal.multiply(CENTO).intValue());
   }

   public String asString() {
      return String.valueOf(percentuale);
   }

   public int asInt() {
      return percentuale;
   }

   public static Percentuale computePercentuale(int divisor, int quotient) {

      if (quotient == 0) {
         return new Percentuale(0);
      }
      return new Percentuale(new Double((double) divisor / (double) quotient * 100).intValue());
   }

   @Override
   public int hashCode() {
      final int prime = 31;
      int result = 1;
      result = prime * result + percentuale;
      return result;
   }

   @Override
   public boolean equals(Object obj) {
      if (this == obj) {
         return true;
      }
      if (obj == null) {
         return false;
      }
      if (getClass() != obj.getClass()) {
         return false;
      }
      Percentuale other = (Percentuale) obj;
      if (percentuale != other.percentuale) {
         return false;
      }
      return true;
   }

}
