package net.phaedra.commons.values;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.*;

import org.apache.commons.lang.time.DateUtils;

public class Time implements Serializable, Comparable {

   private Date date;

   public Time() {
      setTruncatedDate(Calendar.getInstance().getTime());
   }

   public Time(int year, int month, int day, int hour, int minute, int second) {
      Calendar calendar = Calendar.getInstance();
      calendar.set(year, month - 1, day, hour, minute, second);
      setTruncatedDate(calendar.getTime());
   }

   public Time(Date date) {
      setTruncatedDate(date);
   }

   private void setTruncatedDate(Date newDate) {
      Calendar calendar = asCalendar(newDate);
      Calendar truncatedCalendar = DateUtils.truncate(calendar, Calendar.SECOND);
      this.date = truncatedCalendar.getTime();
   }

   @Override
   public int hashCode() {
      return date.hashCode();
   }

   @Override
   public boolean equals(Object obj) {
      if (this == obj)
         return true;
      if (obj == null)
         return false;
      if (getClass() != obj.getClass())
         return false;
      Time other = (Time) obj;
      if (date == null) {
         if (other.date != null)
            return false;
      } else if (!date.equals(other.date))
         return false;
      return true;
   }

   @Override
   public String toString() {
      Calendar calendar = asCalendar(date);
      int humanReadableMonth = calendar.get(Calendar.MONTH) + 1;
      String timeString = calendar.get(Calendar.DAY_OF_MONTH) + "" + humanReadableMonth + ""
            + calendar.get(Calendar.YEAR) + " " + calendar.get(Calendar.HOUR_OF_DAY) + " "
            + calendar.get(Calendar.MINUTE) + "" + calendar.get(Calendar.SECOND);
      return timeString;
   }

   public Date asDate() {
      return date;
   }

   public static Time convertDateToTime(Date date) {
      Calendar calendar = asCalendar(date);
      int day = calendar.get(Calendar.DAY_OF_MONTH);
      int month = calendar.get(Calendar.MONTH) + 1;
      int year = calendar.get(Calendar.YEAR);
      int hour = calendar.get(Calendar.HOUR_OF_DAY);
      int minute = calendar.get(Calendar.MINUTE);
      int second = calendar.get(Calendar.SECOND);
      Time time = new Time(year, month, day, hour, minute, second);
      return time;
   }

   public int getHours() {
      Calendar calendar = asCalendar(date);
      int hour = calendar.get(Calendar.HOUR_OF_DAY);
      return hour;
   }

   private static Calendar asCalendar(Date date) {
      Calendar calendar = Calendar.getInstance();
      calendar.setTime(date);
      return calendar;
   }

   public int getMinutes() {
      Calendar calendar = asCalendar(date);
      int minutes = calendar.get(Calendar.MINUTE);
      return minutes;
   }

   public int getSeconds() {
      Calendar calendar = asCalendar(date);
      int seconds = calendar.get(Calendar.SECOND);
      return seconds;
   }

   @Override
   public int compareTo(Object o) {
      Time other = (Time) o;
      return date.compareTo(other.asDate());
   }

   public static Time yesterday() {
      return Day.today().addDays(-1).asTime();
   }

   public String format(String format) {
      SimpleDateFormat sdf = new SimpleDateFormat(format, Locale.ITALIAN);
      return sdf.format(asDate());
   }
}
